#Pragma once

'' The following symbols have been renamed:
''     constant ENET_HOST_BROADCAST_ alias "ENET_HOST_BROADCAST"
''     constant ENET_PEER_PING_INTERVAL_ alias "ENET_PEER_PING_INTERVAL"

extern "C"

type _ENetEvent as _ENetEvent_
type _ENetHost as _ENetHost_
type _ENetPacket as _ENetPacket_

#Ifdef __FB_WIN32__
	#include "win/winsock2.bi"

	type ENetSocket as SOCKET

	#define ENET_SOCKET_NULL INVALID_SOCKET
#else
	#include "crt/sys/time.h"
	#include "crt/sys/types.h"
	#include "crt/sys/socket.h"
	#include "crt/netinet/in.h"
	#include "crt/unistd.h"

	type ENetSocket as long

	const ENET_SOCKET_NULL = -1
#endif

#define ENET_HOST_TO_NET_16(value) htons(value)
#define ENET_HOST_TO_NET_32(value) htonl(value)
#define ENET_NET_TO_HOST_16(value) ntohs(value)
#define ENET_NET_TO_HOST_32(value) ntohl(value)

type ENetBuffer
	#ifdef __FB_LINUX__
		data as any ptr
	#endif

	dataLength as uinteger

	#ifdef __FB_WIN32__
		data as any ptr
	#endif
end type

type ENetSocketSet as fd_set

#define ENET_SOCKETSET_EMPTY(sockset) FD_ZERO(@(sockset))
#define ENET_SOCKETSET_ADD(sockset, socket) FD_SET(socket, @(sockset))
#define ENET_SOCKETSET_REMOVE(sockset, socket) FD_CLR(socket, @(sockset))
#define ENET_SOCKETSET_CHECK(sockset, socket) FD_ISSET(socket, @(sockset))

type enet_uint8 as ubyte
type enet_uint16 as ushort
type enet_uint32 as ulong

enum
	ENET_PROTOCOL_MINIMUM_MTU = 576
	ENET_PROTOCOL_MAXIMUM_MTU = 4096
	ENET_PROTOCOL_MAXIMUM_PACKET_COMMANDS = 32
	ENET_PROTOCOL_MINIMUM_WINDOW_SIZE = 4096
	ENET_PROTOCOL_MAXIMUM_WINDOW_SIZE = 65536
	ENET_PROTOCOL_MINIMUM_CHANNEL_COUNT = 1
	ENET_PROTOCOL_MAXIMUM_CHANNEL_COUNT = 255
	ENET_PROTOCOL_MAXIMUM_PEER_ID = &hFFF
	ENET_PROTOCOL_MAXIMUM_FRAGMENT_COUNT = 1024 * 1024
end enum

type _ENetProtocolCommand as long
enum
	ENET_PROTOCOL_COMMAND_NONE = 0
	ENET_PROTOCOL_COMMAND_ACKNOWLEDGE = 1
	ENET_PROTOCOL_COMMAND_CONNECT = 2
	ENET_PROTOCOL_COMMAND_VERIFY_CONNECT = 3
	ENET_PROTOCOL_COMMAND_DISCONNECT = 4
	ENET_PROTOCOL_COMMAND_PING = 5
	ENET_PROTOCOL_COMMAND_SEND_RELIABLE = 6
	ENET_PROTOCOL_COMMAND_SEND_UNRELIABLE = 7
	ENET_PROTOCOL_COMMAND_SEND_FRAGMENT = 8
	ENET_PROTOCOL_COMMAND_SEND_UNSEQUENCED = 9
	ENET_PROTOCOL_COMMAND_BANDWIDTH_LIMIT = 10
	ENET_PROTOCOL_COMMAND_THROTTLE_CONFIGURE = 11
	ENET_PROTOCOL_COMMAND_SEND_UNRELIABLE_FRAGMENT = 12
	ENET_PROTOCOL_COMMAND_COUNT = 13
	ENET_PROTOCOL_COMMAND_MASK = &hF
end enum

type ENetProtocolCommand as _ENetProtocolCommand

type _ENetProtocolFlag as long
enum
	ENET_PROTOCOL_COMMAND_FLAG_ACKNOWLEDGE = 1 shl 7
	ENET_PROTOCOL_COMMAND_FLAG_UNSEQUENCED = 1 shl 6
	ENET_PROTOCOL_HEADER_FLAG_COMPRESSED = 1 shl 14
	ENET_PROTOCOL_HEADER_FLAG_SENT_TIME = 1 shl 15
	ENET_PROTOCOL_HEADER_FLAG_MASK = ENET_PROTOCOL_HEADER_FLAG_COMPRESSED or ENET_PROTOCOL_HEADER_FLAG_SENT_TIME
	ENET_PROTOCOL_HEADER_SESSION_MASK = 3 shl 12
	ENET_PROTOCOL_HEADER_SESSION_SHIFT = 12
end enum

type ENetProtocolFlag as _ENetProtocolFlag

type _ENetProtocolHeader field = 1
	peerID as enet_uint16
	sentTime as enet_uint16
end type

type ENetProtocolHeader as _ENetProtocolHeader

type _ENetProtocolCommandHeader field = 1
	command as enet_uint8
	channelID as enet_uint8
	reliableSequenceNumber as enet_uint16
end type

type ENetProtocolCommandHeader as _ENetProtocolCommandHeader

type _ENetProtocolAcknowledge field = 1
	header as ENetProtocolCommandHeader
	receivedReliableSequenceNumber as enet_uint16
	receivedSentTime as enet_uint16
end type

type ENetProtocolAcknowledge as _ENetProtocolAcknowledge

type _ENetProtocolConnect field = 1
	header as ENetProtocolCommandHeader
	outgoingPeerID as enet_uint16
	incomingSessionID as enet_uint8
	outgoingSessionID as enet_uint8
	mtu as enet_uint32
	windowSize as enet_uint32
	channelCount as enet_uint32
	incomingBandwidth as enet_uint32
	outgoingBandwidth as enet_uint32
	packetThrottleInterval as enet_uint32
	packetThrottleAcceleration as enet_uint32
	packetThrottleDeceleration as enet_uint32
	connectID as enet_uint32
	data as enet_uint32
end type

type ENetProtocolConnect as _ENetProtocolConnect

type _ENetProtocolVerifyConnect field = 1
	header as ENetProtocolCommandHeader
	outgoingPeerID as enet_uint16
	incomingSessionID as enet_uint8
	outgoingSessionID as enet_uint8
	mtu as enet_uint32
	windowSize as enet_uint32
	channelCount as enet_uint32
	incomingBandwidth as enet_uint32
	outgoingBandwidth as enet_uint32
	packetThrottleInterval as enet_uint32
	packetThrottleAcceleration as enet_uint32
	packetThrottleDeceleration as enet_uint32
	connectID as enet_uint32
end type

type ENetProtocolVerifyConnect as _ENetProtocolVerifyConnect

type _ENetProtocolBandwidthLimit field = 1
	header as ENetProtocolCommandHeader
	incomingBandwidth as enet_uint32
	outgoingBandwidth as enet_uint32
end type

type ENetProtocolBandwidthLimit as _ENetProtocolBandwidthLimit

type _ENetProtocolThrottleConfigure field = 1
	header as ENetProtocolCommandHeader
	packetThrottleInterval as enet_uint32
	packetThrottleAcceleration as enet_uint32
	packetThrottleDeceleration as enet_uint32
end type

type ENetProtocolThrottleConfigure as _ENetProtocolThrottleConfigure

type _ENetProtocolDisconnect field = 1
	header as ENetProtocolCommandHeader
	data as enet_uint32
end type

type ENetProtocolDisconnect as _ENetProtocolDisconnect

type _ENetProtocolPing field = 1
	header as ENetProtocolCommandHeader
end type

type ENetProtocolPing as _ENetProtocolPing

type _ENetProtocolSendReliable field = 1
	header as ENetProtocolCommandHeader
	dataLength as enet_uint16
end type

type ENetProtocolSendReliable as _ENetProtocolSendReliable

type _ENetProtocolSendUnreliable field = 1
	header as ENetProtocolCommandHeader
	unreliableSequenceNumber as enet_uint16
	dataLength as enet_uint16
end type

type ENetProtocolSendUnreliable as _ENetProtocolSendUnreliable

type _ENetProtocolSendUnsequenced field = 1
	header as ENetProtocolCommandHeader
	unsequencedGroup as enet_uint16
	dataLength as enet_uint16
end type

type ENetProtocolSendUnsequenced as _ENetProtocolSendUnsequenced

type _ENetProtocolSendFragment field = 1
	header as ENetProtocolCommandHeader
	startSequenceNumber as enet_uint16
	dataLength as enet_uint16
	fragmentCount as enet_uint32
	fragmentNumber as enet_uint32
	totalLength as enet_uint32
	fragmentOffset as enet_uint32
end type

type ENetProtocolSendFragment as _ENetProtocolSendFragment

union _ENetProtocol field = 1
	header as ENetProtocolCommandHeader
	acknowledge as ENetProtocolAcknowledge
	connect as ENetProtocolConnect
	verifyConnect as ENetProtocolVerifyConnect
	disconnect as ENetProtocolDisconnect
	ping as ENetProtocolPing
	sendReliable as ENetProtocolSendReliable
	sendUnreliable as ENetProtocolSendUnreliable
	sendUnsequenced as ENetProtocolSendUnsequenced
	sendFragment as ENetProtocolSendFragment
	bandwidthLimit as ENetProtocolBandwidthLimit
	throttleConfigure as ENetProtocolThrottleConfigure
end union

type ENetProtocol as _ENetProtocol


type _ENetListNode
	next as _ENetListNode ptr
	previous as _ENetListNode ptr
end type

type ENetListNode as _ENetListNode
type ENetListIterator as ENetListNode ptr

type _ENetList
	sentinel as ENetListNode
end type

type ENetList as _ENetList

declare sub enet_list_clear(byval as ENetList ptr)
declare function enet_list_insert(byval as ENetListIterator, byval as any ptr) as ENetListIterator
declare function enet_list_remove(byval as ENetListIterator) as any ptr
declare function enet_list_move(byval as ENetListIterator, byval as any ptr, byval as any ptr) as ENetListIterator
declare function enet_list_size(byval as ENetList ptr) as uinteger

#define enet_list_begin(list) (list)->sentinel.next
#define enet_list_end(list) (@(list)->sentinel)
#define enet_list_empty(list) (enet_list_begin(list) = enet_list_end(list))
#define enet_list_next(iterator) (iterator)->next
#define enet_list_previous(iterator) (iterator)->previous
#define enet_list_front(list) cptr(any ptr, (list)->sentinel.next)
#define enet_list_back(list) cptr(any ptr, (list)->sentinel.previous)

type _ENetCallbacks
	malloc as function(byval size as uinteger) as any ptr
	free as sub(byval memory as any ptr)
	no_memory as sub()
end type

type ENetCallbacks as _ENetCallbacks

declare function enet_malloc(byval as uinteger) as any ptr
declare sub enet_free(byval as any ptr)

const ENET_VERSION_MAJOR = 1
const ENET_VERSION_MINOR = 3
const ENET_VERSION_PATCH = 12

#define ENET_VERSION_CREATE(major, minor, patch) ((((major) shl 16) or ((minor) shl 8)) or (patch))
#define ENET_VERSION_GET_MAJOR(version) (((version) shr 16) and &hFF)
#define ENET_VERSION_GET_MINOR(version) (((version) shr 8) and &hFF)
#define ENET_VERSION_GET_PATCH(version) ((version) and &hFF)
#define ENET_VERSION ENET_VERSION_CREATE(ENET_VERSION_MAJOR, ENET_VERSION_MINOR, ENET_VERSION_PATCH)

type ENetVersion as enet_uint32

type _ENetSocketType as long
enum
	ENET_SOCKET_TYPE_STREAM = 1
	ENET_SOCKET_TYPE_DATAGRAM = 2
end enum

type ENetSocketType as _ENetSocketType

type _ENetSocketWait as long
enum
	ENET_SOCKET_WAIT_NONE = 0
	ENET_SOCKET_WAIT_SEND = 1 shl 0
	ENET_SOCKET_WAIT_RECEIVE = 1 shl 1
	ENET_SOCKET_WAIT_INTERRUPT = 1 shl 2
end enum

type ENetSocketWait as _ENetSocketWait

type _ENetSocketOption as long
enum
	ENET_SOCKOPT_NONBLOCK = 1
	ENET_SOCKOPT_BROADCAST = 2
	ENET_SOCKOPT_RCVBUF = 3
	ENET_SOCKOPT_SNDBUF = 4
	ENET_SOCKOPT_REUSEADDR = 5
	ENET_SOCKOPT_RCVTIMEO = 6
	ENET_SOCKOPT_SNDTIMEO = 7
	ENET_SOCKOPT_ERROR = 8
	ENET_SOCKOPT_NODELAY = 9
end enum

type ENetSocketOption as _ENetSocketOption

type _ENetSocketShutdown as long
enum
	ENET_SOCKET_SHUTDOWN_READ = 0
	ENET_SOCKET_SHUTDOWN_WRITE = 1
	ENET_SOCKET_SHUTDOWN_READ_WRITE = 2
end enum

type ENetSocketShutdown as _ENetSocketShutdown

const ENET_HOST_ANY = 0
const ENET_HOST_BROADCAST_ = &hFFFFFFFF
const ENET_PORT_ANY = 0

type _ENetAddress
	host as enet_uint32
	port as enet_uint16
end type

type ENetAddress as _ENetAddress

type _ENetPacketFlag as long
enum
	ENET_PACKET_FLAG_RELIABLE = 1 shl 0
	ENET_PACKET_FLAG_UNSEQUENCED = 1 shl 1
	ENET_PACKET_FLAG_NO_ALLOCATE = 1 shl 2
	ENET_PACKET_FLAG_UNRELIABLE_FRAGMENT = 1 shl 3
	ENET_PACKET_FLAG_SENT = 1 shl 8
end enum

type ENetPacketFlag as _ENetPacketFlag
type ENetPacketFreeCallback as sub(byval as _ENetPacket ptr)

type _ENetPacket_
	referenceCount as uinteger
	flags as enet_uint32
	data as enet_uint8 ptr
	dataLength as uinteger
	freeCallback as ENetPacketFreeCallback
	userData as any ptr
end type

type ENetPacket as _ENetPacket

type _ENetAcknowledgement
	acknowledgementList as ENetListNode
	sentTime as enet_uint32
	command as ENetProtocol
end type

type ENetAcknowledgement as _ENetAcknowledgement

type _ENetOutgoingCommand
	outgoingCommandList as ENetListNode
	reliableSequenceNumber as enet_uint16
	unreliableSequenceNumber as enet_uint16
	sentTime as enet_uint32
	roundTripTimeout as enet_uint32
	roundTripTimeoutLimit as enet_uint32
	fragmentOffset as enet_uint32
	fragmentLength as enet_uint16
	sendAttempts as enet_uint16
	command as ENetProtocol
	packet as ENetPacket ptr
end type

type ENetOutgoingCommand as _ENetOutgoingCommand

type _ENetIncomingCommand
	incomingCommandList as ENetListNode
	reliableSequenceNumber as enet_uint16
	unreliableSequenceNumber as enet_uint16
	command as ENetProtocol
	fragmentCount as enet_uint32
	fragmentsRemaining as enet_uint32
	fragments as enet_uint32 ptr
	packet as ENetPacket ptr
end type

type ENetIncomingCommand as _ENetIncomingCommand

type _ENetPeerState as long
enum
	ENET_PEER_STATE_DISCONNECTED = 0
	ENET_PEER_STATE_CONNECTING = 1
	ENET_PEER_STATE_ACKNOWLEDGING_CONNECT = 2
	ENET_PEER_STATE_CONNECTION_PENDING = 3
	ENET_PEER_STATE_CONNECTION_SUCCEEDED = 4
	ENET_PEER_STATE_CONNECTED = 5
	ENET_PEER_STATE_DISCONNECT_LATER = 6
	ENET_PEER_STATE_DISCONNECTING = 7
	ENET_PEER_STATE_ACKNOWLEDGING_DISCONNECT = 8
	ENET_PEER_STATE_ZOMBIE = 9
end enum

type ENetPeerState as _ENetPeerState

#define ENET_BUFFER_MAXIMUM (1 + (2 * ENET_PROTOCOL_MAXIMUM_PACKET_COMMANDS))

enum
	ENET_HOST_RECEIVE_BUFFER_SIZE = 256 * 1024
	ENET_HOST_SEND_BUFFER_SIZE = 256 * 1024
	ENET_HOST_BANDWIDTH_THROTTLE_INTERVAL = 1000
	ENET_HOST_DEFAULT_MTU = 1400
	ENET_HOST_DEFAULT_MAXIMUM_PACKET_SIZE = (32 * 1024) * 1024
	ENET_HOST_DEFAULT_MAXIMUM_WAITING_DATA = (32 * 1024) * 1024
	ENET_PEER_DEFAULT_ROUND_TRIP_TIME = 500
	ENET_PEER_DEFAULT_PACKET_THROTTLE = 32
	ENET_PEER_PACKET_THROTTLE_SCALE = 32
	ENET_PEER_PACKET_THROTTLE_COUNTER = 7
	ENET_PEER_PACKET_THROTTLE_ACCELERATION = 2
	ENET_PEER_PACKET_THROTTLE_DECELERATION = 2
	ENET_PEER_PACKET_THROTTLE_INTERVAL = 5000
	ENET_PEER_PACKET_LOSS_SCALE = 1 shl 16
	ENET_PEER_PACKET_LOSS_INTERVAL = 10000
	ENET_PEER_WINDOW_SIZE_SCALE = 64 * 1024
	ENET_PEER_TIMEOUT_LIMIT = 32
	ENET_PEER_TIMEOUT_MINIMUM = 5000
	ENET_PEER_TIMEOUT_MAXIMUM = 30000
	ENET_PEER_PING_INTERVAL_ = 500
	ENET_PEER_UNSEQUENCED_WINDOWS = 64
	ENET_PEER_UNSEQUENCED_WINDOW_SIZE = 1024
	ENET_PEER_FREE_UNSEQUENCED_WINDOWS = 32
	ENET_PEER_RELIABLE_WINDOWS = 16
	ENET_PEER_RELIABLE_WINDOW_SIZE = &h1000
	ENET_PEER_FREE_RELIABLE_WINDOWS = 8
end enum

type _ENetChannel
	outgoingReliableSequenceNumber as enet_uint16
	outgoingUnreliableSequenceNumber as enet_uint16
	usedReliableWindows as enet_uint16
	reliableWindows(0 to ENET_PEER_RELIABLE_WINDOWS - 1) as enet_uint16
	incomingReliableSequenceNumber as enet_uint16
	incomingUnreliableSequenceNumber as enet_uint16
	incomingReliableCommands as ENetList
	incomingUnreliableCommands as ENetList
end type

type ENetChannel as _ENetChannel

type _ENetPeer
	dispatchList as ENetListNode
	host as _ENetHost ptr
	outgoingPeerID as enet_uint16
	incomingPeerID as enet_uint16
	connectID as enet_uint32
	outgoingSessionID as enet_uint8
	incomingSessionID as enet_uint8
	address as ENetAddress
	data as any ptr
	state as ENetPeerState
	channels as ENetChannel ptr
	channelCount as uinteger
	incomingBandwidth as enet_uint32
	outgoingBandwidth as enet_uint32
	incomingBandwidthThrottleEpoch as enet_uint32
	outgoingBandwidthThrottleEpoch as enet_uint32
	incomingDataTotal as enet_uint32
	outgoingDataTotal as enet_uint32
	lastSendTime as enet_uint32
	lastReceiveTime as enet_uint32
	nextTimeout as enet_uint32
	earliestTimeout as enet_uint32
	packetLossEpoch as enet_uint32
	packetsSent as enet_uint32
	packetsLost as enet_uint32
	packetLoss as enet_uint32
	packetLossVariance as enet_uint32
	packetThrottle as enet_uint32
	packetThrottleLimit as enet_uint32
	packetThrottleCounter as enet_uint32
	packetThrottleEpoch as enet_uint32
	packetThrottleAcceleration as enet_uint32
	packetThrottleDeceleration as enet_uint32
	packetThrottleInterval as enet_uint32
	pingInterval as enet_uint32
	timeoutLimit as enet_uint32
	timeoutMinimum as enet_uint32
	timeoutMaximum as enet_uint32
	lastRoundTripTime as enet_uint32
	lowestRoundTripTime as enet_uint32
	lastRoundTripTimeVariance as enet_uint32
	highestRoundTripTimeVariance as enet_uint32
	roundTripTime as enet_uint32
	roundTripTimeVariance as enet_uint32
	mtu as enet_uint32
	windowSize as enet_uint32
	reliableDataInTransit as enet_uint32
	outgoingReliableSequenceNumber as enet_uint16
	acknowledgements as ENetList
	sentReliableCommands as ENetList
	sentUnreliableCommands as ENetList
	outgoingReliableCommands as ENetList
	outgoingUnreliableCommands as ENetList
	dispatchedCommands as ENetList
	needsDispatch as long
	incomingUnsequencedGroup as enet_uint16
	outgoingUnsequencedGroup as enet_uint16
	unsequencedWindow(0 to (ENET_PEER_UNSEQUENCED_WINDOW_SIZE / 32) - 1) as enet_uint32
	eventData as enet_uint32
	totalWaitingData as uinteger
end type

type ENetPeer as _ENetPeer

type _ENetCompressor
	context as any ptr
	compress as function(byval context as any ptr, byval inBuffers as const ENetBuffer ptr, byval inBufferCount as uinteger, byval inLimit as uinteger, byval outData as enet_uint8 ptr, byval outLimit as uinteger) as uinteger
	decompress as function(byval context as any ptr, byval inData as const enet_uint8 ptr, byval inLimit as uinteger, byval outData as enet_uint8 ptr, byval outLimit as uinteger) as uinteger
	destroy as sub(byval context as any ptr)
end type

type ENetCompressor as _ENetCompressor
type ENetChecksumCallback as function(byval buffers as const ENetBuffer ptr, byval bufferCount as uinteger) as enet_uint32
type ENetInterceptCallback as function(byval host as _ENetHost ptr, byval event as _ENetEvent ptr) as long

type _ENetHost_
	socket as ENetSocket
	address as ENetAddress
	incomingBandwidth as enet_uint32
	outgoingBandwidth as enet_uint32
	bandwidthThrottleEpoch as enet_uint32
	mtu as enet_uint32
	randomSeed as enet_uint32
	recalculateBandwidthLimits as long
	peers as ENetPeer ptr
	peerCount as uinteger
	channelLimit as uinteger
	serviceTime as enet_uint32
	dispatchQueue as ENetList
	continueSending as long
	packetSize as uinteger
	headerFlags as enet_uint16
	commands(0 to ENET_PROTOCOL_MAXIMUM_PACKET_COMMANDS - 1) as ENetProtocol
	commandCount as uinteger
	buffers(0 to (1 + (2 * ENET_PROTOCOL_MAXIMUM_PACKET_COMMANDS)) - 1) as ENetBuffer
	bufferCount as uinteger
	checksum as ENetChecksumCallback
	compressor as ENetCompressor
	packetData(0 to 1, 0 to ENET_PROTOCOL_MAXIMUM_MTU - 1) as enet_uint8
	receivedAddress as ENetAddress
	receivedData as enet_uint8 ptr
	receivedDataLength as uinteger
	totalSentData as enet_uint32
	totalSentPackets as enet_uint32
	totalReceivedData as enet_uint32
	totalReceivedPackets as enet_uint32
	intercept as ENetInterceptCallback
	connectedPeers as uinteger
	bandwidthLimitedPeers as uinteger
	duplicatePeers as uinteger
	maximumPacketSize as uinteger
	maximumWaitingData as uinteger
end type

type ENetHost as _ENetHost

type _ENetEventType as long
enum
	ENET_EVENT_TYPE_NONE = 0
	ENET_EVENT_TYPE_CONNECT = 1
	ENET_EVENT_TYPE_DISCONNECT = 2
	ENET_EVENT_TYPE_RECEIVE = 3
end enum

type ENetEventType as _ENetEventType

type _ENetEvent_
	as ENetEventType type
	peer as ENetPeer ptr
	channelID as enet_uint8
	data as enet_uint32
	packet as ENetPacket ptr
end type

type ENetEvent as _ENetEvent

declare function enet_initialize() as long
declare function enet_initialize_with_callbacks(byval version as ENetVersion, byval inits as const ENetCallbacks ptr) as long
declare sub enet_deinitialize()
declare function enet_linked_version() as ENetVersion
declare function enet_time_get() as enet_uint32
declare sub enet_time_set(byval as enet_uint32)
declare function enet_socket_create(byval as ENetSocketType) as ENetSocket
declare function enet_socket_bind(byval as ENetSocket, byval as const ENetAddress ptr) as long
declare function enet_socket_get_address(byval as ENetSocket, byval as ENetAddress ptr) as long
declare function enet_socket_listen(byval as ENetSocket, byval as long) as long
declare function enet_socket_accept(byval as ENetSocket, byval as ENetAddress ptr) as ENetSocket
declare function enet_socket_connect(byval as ENetSocket, byval as const ENetAddress ptr) as long
declare function enet_socket_send(byval as ENetSocket, byval as const ENetAddress ptr, byval as const ENetBuffer ptr, byval as uinteger) as long
declare function enet_socket_receive(byval as ENetSocket, byval as ENetAddress ptr, byval as ENetBuffer ptr, byval as uinteger) as long
declare function enet_socket_wait(byval as ENetSocket, byval as enet_uint32 ptr, byval as enet_uint32) as long
declare function enet_socket_set_option(byval as ENetSocket, byval as ENetSocketOption, byval as long) as long
declare function enet_socket_get_option(byval as ENetSocket, byval as ENetSocketOption, byval as long ptr) as long
declare function enet_socket_shutdown(byval as ENetSocket, byval as ENetSocketShutdown) as long
declare sub enet_socket_destroy(byval as ENetSocket)
declare function enet_socketset_select(byval as ENetSocket, byval as ENetSocketSet ptr, byval as ENetSocketSet ptr, byval as enet_uint32) as long
declare function enet_address_set_host(byval address as ENetAddress ptr, byval hostName as const zstring ptr) as long
declare function enet_address_get_host_ip(byval address as const ENetAddress ptr, byval hostName as zstring ptr, byval nameLength as uinteger) as long
declare function enet_address_get_host(byval address as const ENetAddress ptr, byval hostName as zstring ptr, byval nameLength as uinteger) as long
declare function enet_packet_create(byval as const any ptr, byval as uinteger, byval as enet_uint32) as ENetPacket ptr
declare sub enet_packet_destroy(byval as ENetPacket ptr)
declare function enet_packet_resize(byval as ENetPacket ptr, byval as uinteger) as long
declare function enet_crc32(byval as const ENetBuffer ptr, byval as uinteger) as enet_uint32
declare function enet_host_create(byval as const ENetAddress ptr, byval as uinteger, byval as uinteger, byval as enet_uint32, byval as enet_uint32) as ENetHost ptr
declare sub enet_host_destroy(byval as ENetHost ptr)
declare function enet_host_connect(byval as ENetHost ptr, byval as const ENetAddress ptr, byval as uinteger, byval as enet_uint32) as ENetPeer ptr
declare function enet_host_check_events(byval as ENetHost ptr, byval as ENetEvent ptr) as long
declare function enet_host_service(byval as ENetHost ptr, byval as ENetEvent ptr, byval as enet_uint32) as long
declare sub enet_host_flush(byval as ENetHost ptr)
declare sub enet_host_broadcast(byval as ENetHost ptr, byval as enet_uint8, byval as ENetPacket ptr)
declare sub enet_host_compress(byval as ENetHost ptr, byval as const ENetCompressor ptr)
declare function enet_host_compress_with_range_coder(byval host as ENetHost ptr) as long
declare sub enet_host_channel_limit(byval as ENetHost ptr, byval as uinteger)
declare sub enet_host_bandwidth_limit(byval as ENetHost ptr, byval as enet_uint32, byval as enet_uint32)
declare sub enet_host_bandwidth_throttle(byval as ENetHost ptr)
declare function enet_host_random_seed() as enet_uint32
declare function enet_peer_send(byval as ENetPeer ptr, byval as enet_uint8, byval as ENetPacket ptr) as long
declare function enet_peer_receive(byval as ENetPeer ptr, byval channelID as enet_uint8 ptr) as ENetPacket ptr
declare sub enet_peer_ping(byval as ENetPeer ptr)
declare sub enet_peer_ping_interval(byval as ENetPeer ptr, byval as enet_uint32)
declare sub enet_peer_timeout(byval as ENetPeer ptr, byval as enet_uint32, byval as enet_uint32, byval as enet_uint32)
declare sub enet_peer_reset(byval as ENetPeer ptr)
declare sub enet_peer_disconnect(byval as ENetPeer ptr, byval as enet_uint32)
declare sub enet_peer_disconnect_now(byval as ENetPeer ptr, byval as enet_uint32)
declare sub enet_peer_disconnect_later(byval as ENetPeer ptr, byval as enet_uint32)
declare sub enet_peer_throttle_configure(byval as ENetPeer ptr, byval as enet_uint32, byval as enet_uint32, byval as enet_uint32)
declare function enet_peer_throttle(byval as ENetPeer ptr, byval as enet_uint32) as long
declare sub enet_peer_reset_queues(byval as ENetPeer ptr)
declare sub enet_peer_setup_outgoing_command(byval as ENetPeer ptr, byval as ENetOutgoingCommand ptr)
declare function enet_peer_queue_outgoing_command(byval as ENetPeer ptr, byval as const ENetProtocol ptr, byval as ENetPacket ptr, byval as enet_uint32, byval as enet_uint16) as ENetOutgoingCommand ptr
declare function enet_peer_queue_incoming_command(byval as ENetPeer ptr, byval as const ENetProtocol ptr, byval as const any ptr, byval as uinteger, byval as enet_uint32, byval as enet_uint32) as ENetIncomingCommand ptr
declare function enet_peer_queue_acknowledgement(byval as ENetPeer ptr, byval as const ENetProtocol ptr, byval as enet_uint16) as ENetAcknowledgement ptr
declare sub enet_peer_dispatch_incoming_unreliable_commands(byval as ENetPeer ptr, byval as ENetChannel ptr)
declare sub enet_peer_dispatch_incoming_reliable_commands(byval as ENetPeer ptr, byval as ENetChannel ptr)
declare sub enet_peer_on_connect(byval as ENetPeer ptr)
declare sub enet_peer_on_disconnect(byval as ENetPeer ptr)
declare function enet_range_coder_create() as any ptr
declare sub enet_range_coder_destroy(byval as any ptr)
declare function enet_range_coder_compress(byval as any ptr, byval as const ENetBuffer ptr, byval as uinteger, byval as uinteger, byval as enet_uint8 ptr, byval as uinteger) as uinteger
declare function enet_range_coder_decompress(byval as any ptr, byval as const enet_uint8 ptr, byval as uinteger, byval as enet_uint8 ptr, byval as uinteger) as uinteger
declare function enet_protocol_command_size(byval as enet_uint8) as uinteger

const ENET_TIME_OVERFLOW = 86400000

#define ENET_TIME_LESS(a, b) (((a) - (b)) >= ENET_TIME_OVERFLOW)
#define ENET_TIME_GREATER(a, b) (((b) - (a)) >= ENET_TIME_OVERFLOW)
#define ENET_TIME_LESS_EQUAL(a, b) (ENET_TIME_GREATER(a, b) = 0)
#define ENET_TIME_GREATER_EQUAL(a, b) (ENET_TIME_LESS(a, b) = 0)
#define ENET_TIME_DIFFERENCE(a, b) iif(((a) - (b)) >= ENET_TIME_OVERFLOW, (b) - (a), (a) - (b))

#define ENET_MAX(x, y) iif((x) > (y), (x), (y))
#define ENET_MIN(x, y) iif((x) < (y), (x), (y))
end extern
